@@ -2,6 +2,7 @@ import fnmatch
 import os
 import shutil
 import stat
+from distutils.dir_util import copy_tree
 
 
 # -----------------------------------------------------------------------------
@@ -158,37 +159,38 @@ def find_files(path, pattern, recursive=False):
 
     results = []
 
-    if recursive:
-        for root, _, files in os.walk(path, topdown=False):
-            for name in files:
-                base_path = os.path.join(root, name)
-                filename = os.path.basename(base_path)
+    if dir_exists(path):
+        if recursive:
+            for root, _, files in os.walk(path, topdown=False):
+                for name in files:
+                    base_path = os.path.join(root, name)
+                    filename = os.path.basename(base_path)
 
-                if isinstance(pattern, list):
-                    if [
-                        pattern_item
-                        for pattern_item in pattern
-                        if fnmatch.fnmatch(filename, pattern_item)
-                    ]:
+                    if isinstance(pattern, list):
+                        if [
+                            pattern_item
+                            for pattern_item in pattern
+                            if fnmatch.fnmatch(filename, pattern_item)
+                        ]:
+                            results.append(base_path)
+                    elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
                         results.append(base_path)
-                elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
-                    results.append(base_path)
-    else:
-        for item in os.listdir(path):
-            base_path = os.path.join(path, item)
+        else:
+            for item in os.listdir(path):
+                base_path = os.path.join(path, item)
 
-            if os.path.isfile(base_path):
-                filename = os.path.basename(base_path)
+                if os.path.isfile(base_path):
+                    filename = os.path.basename(base_path)
 
-                if isinstance(pattern, list):
-                    if [
-                        pattern_item
-                        for pattern_item in pattern
-                        if fnmatch.fnmatch(filename, pattern_item)
-                    ]:
+                    if isinstance(pattern, list):
+                        if [
+                            pattern_item
+                            for pattern_item in pattern
+                            if fnmatch.fnmatch(filename, pattern_item)
+                        ]:
+                            results.append(base_path)
+                    elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
                         results.append(base_path)
-                elif pattern == "*" or fnmatch.fnmatch(filename, pattern):
-                    results.append(base_path)
 
     return results
 
@@ -213,37 +215,38 @@ def find_dirs(path, pattern, recursive=False):
 
     results = []
 
-    if recursive:
-        for root, dirs, _ in os.walk(path, topdown=False):
-            for name in dirs:
-                base_path = os.path.join(root, name)
-                dirname = os.path.basename(base_path)
+    if dir_exists(path):
+        if recursive:
+            for root, dirs, _ in os.walk(path, topdown=False):
+                for name in dirs:
+                    base_path = os.path.join(root, name)
+                    dirname = os.path.basename(base_path)
 
-                if isinstance(pattern, list):
-                    if [
-                        pattern_item
-                        for pattern_item in pattern
-                        if fnmatch.fnmatch(dirname, pattern_item)
-                    ]:
+                    if isinstance(pattern, list):
+                        if [
+                            pattern_item
+                            for pattern_item in pattern
+                            if fnmatch.fnmatch(dirname, pattern_item)
+                        ]:
+                            results.append(base_path)
+                    elif pattern == "*" or fnmatch.fnmatch(dirname, pattern):
                         results.append(base_path)
-                elif pattern == "*" or fnmatch.fnmatch(dirname, pattern):
-                    results.append(base_path)
-    else:
-        for item in os.listdir(path):
-            base_path = os.path.join(path, item)
+        else:
+            for item in os.listdir(path):
+                base_path = os.path.join(path, item)
 
-            if os.path.isdir(base_path):
-                if isinstance(pattern, list):
-                    if [
-                        pattern_item
-                        for pattern_item in pattern
-                        if fnmatch.fnmatch(item, pattern_item)
-                    ]:
+                if os.path.isdir(base_path):
+                    if isinstance(pattern, list):
+                        if [
+                            pattern_item
+                            for pattern_item in pattern
+                            if fnmatch.fnmatch(item, pattern_item)
+                        ]:
+                            results.append(base_path)
+                    elif pattern == "*" or fnmatch.fnmatch(item, pattern):
                         results.append(base_path)
-                elif pattern == "*" or fnmatch.fnmatch(item, pattern):
-                    results.append(base_path)
 
-    return results
+        return results
 
 
 # -----------------------------------------------------------------------------
@@ -408,6 +411,24 @@ def copy_files(source_path, target_path, pattern, symlinks=True):
                 )
 
 
+# -----------------------------------------------------------------------------
+def copy_all(src_path, dst_path):
+    """
+    Copy all files and directories inside source path to target path, creating the target directory if not exists.
+
+    Arguments:
+        source_path : str
+
+        target_path : str
+
+    Returns:
+        None
+    """
+
+    create_dir(dst_path)
+    copy_tree(src_path, dst_path, update=1)
+
+
 # -----------------------------------------------------------------------------
 def home_dir():
     """
@@ -518,6 +539,8 @@ def copy_dir(src, dst, symlinks=False, ignore=None, ignore_file=None):
         if os.path.isdir(s):
             copy_dir(s, d, symlinks, ignore, ignore_file)
         else:
+            can_copy = True
+
             if os.path.islink(s):
                 if symlinks:
                     if ignore_file is None:
@@ -531,27 +554,25 @@ def copy_dir(src, dst, symlinks=False, ignore=None, ignore_file=None):
 
                         os.symlink(os.readlink(s), d)
 
-                        try:
+                        if hasattr(os, "lchmod"):
                             st = os.lstat(s)
                             mode = stat.S_IMODE(st.st_mode)
                             os.lchmod(d, mode)
-                        except:
-                            # lchmod not available
-                            pass
                 else:
-                    continue
+                    # ignore this symlink
+                    can_copy = False
 
-            if ignore_file is None:
-                ignored_file = False
-            else:
-                ignored_file = ignore_file(s)
+            if can_copy:
+                if ignore_file is None:
+                    ignored_file = False
+                else:
+                    ignored_file = ignore_file(s)
 
-            if not ignored_file:
-                try:
-                    shutil.copy2(s, d)
-                except:
-                    # ignore some errors
-                    pass
+                if not ignored_file:
+                    try:
+                        shutil.copy2(s, d)
+                    except IOError:
+                        pass
 
 
 # -----------------------------------------------------------------------------
